<?php
/**
 * Project: sURI :: Secure URI<br>
 * File:    sURI/Dom.php
 *
 * sURI에서 사용하는 HTML Dom API
 *
 * @category   HTML
 * @package    sURI
 * @subpackage sURI_dom
 * @author     JoungKyun.Kim <http://oops.org>
 * @copyright  (c) 2014 JoungKyun.Kim
 * @license    BSD
 * @version    $Id: Dom.php 59 2014-01-25 10:01:51Z oops $
 * @link       http://pear.oops.org/package/sURI
 * @since      File available since Release 0.1
 * @filesource
 */

/**
 * sURI에서 사용하는 HTML Dom API
 * @package sURI
 */
Class sURI_Dom {
	// {{{ properties
	/**#@+
	 * @access private
	 */
	/**
	 * @var array
	 */
	private $rtags = array ('input', 'select', 'textarea');
	/**
	 * PHP DomDocument resource
	 * @var resource
	 */
	private $dom;
	/**#@-*/
	/**
	 * HTML code valid 체크 여부
	 * @access public
	 * @var boolean
	 */
	public  $validhtml;
	// }}}

	// {{{ (void) sURI_Dom::dom_compile (&$buf, $validhtml = false, $ret = false)
	/**
	 * <form>의 매개변수 이름을 유효한 파라미터 이름으로 인코딩
	 *
	 * <code>
	 * <input type="text" name="name1" value="Multibyte String" valid="STRING">
	 * </code>
	 *
	 * 위의 코드에서 name1은 XX_name_XXX로 변경이 되며, X는 [a-zA-Z0-9]의
	 * 범위에서 무작위로 적용이 된다.
	 *
	 * 컴파일이 완료된 후, 유효한 매개변수는 제거가 된다.
	 *
	 * @access public
	 * @return void|string
	 * @param string  compile string
	 * @param boolean buf 값이 valid html(<doctype>으로 시작해서 </html>로 종료)이면
	 *                이 값을 true로 설정한다. 만약 HTML의 일부분이라면 false로 설정
	 *                하도록 한다. 기본값은 false 이다.
	 * @param boolean (optional) true를 설정하면 컴파일된 결과를 리턴한다.<br>
	 *                false로 설정하면 컴파일된 결과를 출력한다.
	 */
	function dom_compile (&$buf, $validhtml = false, $ret = false) {
		$this->decode_entities ($buf);

		$this->dom = new DOMDocument ();
		$this->dom->encoding = $this->charset;
		$this->dom->formatOutput = true;

		#$this->summary ();
		$this->validhtml = $validhtml;
		$this->load ($buf);

		$this->form_compile ();
		if ( $ret === false )
			echo $this->save ();
		else
			return $this->save ();
	}
	// }}}

	// {{{ (void) sURI::dom_multibyte_decode (&$str)
	/**
	 * 변환된 특수문자를 원 문자로 복구
	 *
	 * @access public
	 * @return void
	 * @param  string
	 */
	function dom_multibyte_decode (&$str) {
		$str = preg_replace_callback (
			'/(&#x([[:alnum:]]{2})([[:alnum:]]{2});)/',
			create_function ('$matches', "
				\$n = chr (hexdec (\$matches[1])) . chr (hexdec (\$matches[2]));
				return iconv ('ucs-2', 'utf-8', \$n);
			"),
			$str
		);
	}
	// }}}

	// {{{ (void) sURI::summary (void)
	function summary () {
		printf ("%-30s : %s\n", 'actualEncoding', $this->dom->actualEncoding);
		printf ("%-30s : %s\n", 'documentURI', $this->dom->documentURI);
		printf ("%-30s : %s\n", 'encoding', $this->dom->encoding);
		printf ("%-30s : %s\n", 'formatOutput', $this->dom->formatOutput);
		printf ("%-30s : %s\n", 'preserveWhiteSpace', $this->dom->preserveWhiteSpace);
		printf ("%-30s : %s\n", 'recover', $this->dom->recover);
		printf ("%-30s : %s\n", 'resolveExternals', $this->dom->resolveExternals);
		printf ("%-30s : %s\n", 'standalone', $this->dom->standalone);
		printf ("%-30s : %s\n", 'substituteEntities', $this->dom->substituteEntities);
		printf ("%-30s : %s\n", 'validateOnParse', $this->dom->validateOnParse);
		printf ("%-30s : %s\n", 'version', $this->dom->version);
		printf ("%-30s : %s\n", 'xmlEncoding', $this->dom->xmlEncoding);
		printf ("%-30s : %s\n", 'xmlVersion', $this->dom->xmlVersion);
	}
	// }}}

	// {{{ (void) sURI::load ($buf, &$stream = false)
	/**
	 * @access public
	 * @return void
	 * @param  string
	 * @param  resource (optional) DOMDocumnet 리소스. DOMDocumnet
	 *                  리소스를 지정하면 클래스의 DOMDocumnet 대시
	 *                  사용한다.
	 */
	function load ($buf, &$stream = false) {
		$this->set_object ($obj, $stream);
		$this->load_check ($buf);
		$loader = ($this->htmlver >= sURI::XHTML10) ? 'loadXML' : 'loadHTML';
		$obj->$loader ($buf);
	}
	// }}}

	// {{{ (void) sURI::load_check (&$buf)
	/**
	 * @access public
	 * @return void
	 * @param  string
	 */
	function load_check (&$buf) {
		if ( $this->validhtml )
			return;

		$buf = preg_replace ('/\r?\n/', '%new-line%', $buf);
		$buf = <<<EOF
<!DOCTYPE html 
     PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="content-type" content="text/html; charset={$this->charset}" />
</head>
<body>
<hr id="sDom start" />
$buf
<hr id="sDom end" />
</body>
</html>
EOF;
	}
	// }}}

	// {{{ (string) sURI::save ($stream = false)
	/**
	 * 변환이 완료된 코드를 반환
	 *
	 * @access public
	 * @return string
	 * @param  resource (optional) DOMDocumnet 리소스. DOMDocumnet
	 *                  리소스를 지정하면 클래스의 DOMDocumnet 대시
	 *                  사용한다.
	 */
	function save ($stream = false) {
		$this->set_object ($obj, $stream);

		$saves = ($this->htmlver >= sURI::XHTML10) ? 'saveXML' : 'saveHTML';

		$obj->xinclude();
		$r = $obj->$saves ();
		if ( ! $this->validhtml ) {
			$r = preg_replace ('!.*<hr id="sDom start"[ /]*>!s', '', $r);
			$r = preg_replace ('!<hr id="sDom end".*!s', '', $r);
			$r = preg_replace ('/%new-line%/', "\n", $r);
		}
		$this->dom_multibyte_decode ($r);

		return $r;
	}
	// }}}

	// {{{ (void) sURI::form_compile (&$stream = false)
	/**
	 * HTML form을 인코딩 한다.
	 *
	 * @access public
	 * @return void
	 * @param  resource (optional) DOMDocumnet 리소스. DOMDocumnet
	 *                  리소스를 지정하면 클래스의 DOMDocumnet 대시
	 *                  사용한다.
	 */
	function form_compile (&$stream = false) {
		$this->set_object ($obj, $stream);
		$forms = $this->entity ('form', $obj);

		/*
		 * several forms
		 */
		foreach ( $forms as $form ) {
			$this->cmethod = strtolower ($form->getAttribute ('method'));
			$this->cmethodv = ($this->cmethod == 'get') ? 'G' : 'P';

			/*
			 * input / option / textarea
			 */
			foreach ( $this->rtags as $rtag ) {
				$tags = $this->entity ($rtag);

				foreach ( $tags as $tag ) {
					$name = $tag->getAttribute ('name');

					if ( ! trim ($name) )
						continue;

					$valid = $tag->getAttribute ('valid');

					if ( ! trim ($valid) ) {
						if ( $rtag == 'textarea' )
							$valid = 'FULL';
						else
							continue;
					}

					$this->unbind ($valid);
					$this->bind ($valid);
					$this->name_enc ($name);

					$tag->setAttribute ('name', $name);
					$tag->removeAttribute ('valid');
				}
			}
		}
	}
	// }}}

	// {{{ (DOMNodeList) sURI::entity ($tag, &$stream = false)
	/**
	 * 지정된 tag 이름을 가진 모든 요소를 검색
	 *
	 * @access public
	 * @return resource DOMNodeList object를 반환
	 * @param  string tag 이름
	 * @param  resource (optional) DOMDocumnet 리소스. DOMDocumnet
	 *                  리소스를 지정하면 클래스의 DOMDocumnet 대시
	 *                  사용한다.
	 */
	function entity ($tag, &$stream = false) {
		$this->set_object ($obj, $stream);
		return $obj->getElementsByTagName ($tag);
	}
	// }}}

	// {{{ (string) sURI::charsetFix ($attr = 'textContent', &$stream = false)
	/**
	 * 지정된 dom parameter의 문자셋을 UTF-8에서 ISO-8859-1로 변경
	 *
	 * @access public
	 * @return string
	 * @param  string 변경할 DOM 파라미터 이름
	 * @param  resource (optional) DOMDocumnet 리소스. DOMDocumnet
	 *                  리소스를 지정하면 클래스의 DOMDocumnet 대시
	 *                  사용한다.
	 */
	function charsetFix ($attr = 'textContent', &$stream = false) {
		$this->set_object ($obj, $stream);
		return iconv ('utf8', 'iso-8859-1', $obj->$attr);
	}
	// }}}

	// {{{ (string) sURI::charsetFix_str ($buf)
	/**
	 * UTF-8 문자열을 ISO-8859-1로 변경
	 *
	 * @access public
	 * @return string
	 * @param string
	 */
	function charsetFix_str ($buf) {
		return iconv ('utf8', 'iso-8859-1', $buf);
	}
	// }}}

	// {{{ (void) sURI::decode_entities (&$buf)
	/**
	 * &nbsp; 를 공백 문자로 변경
	 *
	 * @access public
	 * @return void
	 * @param string 복구를 시도할 문장
	 */
	function decode_entities (&$buf) {
		$r = array ('/&nbsp;/');
		$d = array (' ');

		$buf = preg_replace ($r, $d, $buf);
	}
	// }}}

	// {{{ (void) sURI::set_object (&$obj, &$stream = false)
	/**
	 * DOMDocumnet를 결정
	 *
	 * @access public
	 * @return string
	 * @param  resource 결정된 DOMDocumnet 리소스
	 * @param  resource (optional) DOMDocumnet 리소스. DOMDocumnet
	 *                  리소스를 지정하면 클래스의 DOMDocumnet 대시
	 *                  사용한다.
	 */
	function set_object (&$obj, &$stream = false) {
		$obj = ($stream === false) ? $this->dom : $stream;
	}
	// }}}
}

?>
