<?php
/**
 * Project: sURI :: Secure URI<br>
 * File:    sURI/Parser.php
 *
 * sURI에서 사용하는 HTML 매개변수 파서
 *
 * @category   HTML
 * @package    sURI
 * @subpackage sURI_template
 * @author     JoungKyun.Kim <http://oops.org>
 * @copyright  (c) 2014 JoungKyun.Kim
 * @license    BSD
 * @version    $Id: Parser.php 59 2014-01-25 10:01:51Z oops $
 * @link       http://pear.oops.org/package/sURI
 * @since      File available since Release 0.1
 * @filesource
 */

/**
 * import sURI_Dom class
 */
require_once 'sURI/Dom.php';
/**
 * import sURI_Template class
 */
require_once 'sURI/Template.php';
/**
 * import hash key
 */
require_once 'sURI/hash.php';

/**
 * sURI에서 사용하는 HTML 매개변수 파서
 * @package sURI
 */
Class sURI_Parser extends sURI_Template {
	// {{{ properties
	/**#@+
	 * @access public
	 */
	public $post = null;
	public $get = null;
	/**#@-*/
	/**#@+
	 * @access private
	 */
	private $pmethod = null;
	private $current_key = null;
	private $current_type = null;
	private $parameter_exec_type = null;
	/**#@-*/
	// }}}

	// {{{ (void) sURI_Parser::parameters (void)
	function parameters () {
		/* GET */
		$this->parameter_exec_type = 'get';
		$this->parameters_api ();
		/* POST */
		$this->parameter_exec_type = 'post';
		$this->parameters_api ();
	}
	// }}}

	// {{{ private (vodi) sURI_Parser::parameters_api (void)
	/**
	 * @access private
	 * @return void
	 */
	private function parameters_api () {
		$method = $this->parameter_exec_type;
		$method = ($method == 'post') ? $_POST : $_GET;
		foreach ( $method as $key => $value ) {
			$this->current_key = $key;
			if ( $this->parameters_parse ($key, $value) === false ) {
				echo "<pre>\n";
				echo "ERROR: "; print_r ($this->errstring);
				echo "</pre>";
				exit;
			}

			if ( $this->pmethod == 'P' )
				$this->post->$key = $value;
			else
				$this->get->$key = $value;
		}

		$this->parameter_exec_type = null;
	}
	// }}}

	// {{{ private (void) sURI_Parser::parameters_set_ascii (&$key)
	/**
	 * @access private
	 * @return void
	 * @param string
	 */
	private function parameters_set_ascii (&$key) {
		$this->cmethod = $this->parameter_exec_type;
		$this->cmethodv = ($this->cmethod == 'post') ? 'P' : 'G';
		$this->bind (ASCIIS);
		$this->name_enc ($key);
	}
	// }}}

	// {{{ private (boolean) sURI_Parser::parameters_parse (&$key, &$value)
	/**
	 * @access private
	 * @return boolean
	 * @param string
	 * @param string
	 */
	private function parameters_parse (&$key, &$value) {
		if ( ! preg_match ('/^([^_]+)_(.+)_([^_]+)$/', $key, $matches) ) {
			if ( $this->legacy ) {
				$this->cmethod = $this->parameter_exec_type;
				$this->pmethod = ($this->cmethod == 'post') ? 'P' : 'G';
				return true;
			}

			/*
			 * 2009/01/10 oops@
			 * legacy 모드가 아닐 경우 키가 없는 변수는 ASCII 처리를 한도록 변경
			 *
			$this->set_error ('Invalid Argument format on ' . $key);
			return false;
			 */
			$this->parameters_set_ascii ($key);
			$this->cmethod  = 'get';
			$this->cmethodv = 'G';
			preg_match ('/^([^_]+)_(.+)_([^_]+)$/', $key, $matches);
		}

		$key = $matches[2];

		if ( ($this->current_type = $this->parameters_method ($matches[1])) === false ) {
			$this->set_error ('\'' . $this->current_key . '\' has invalid method');
			return false;
		}

		// remove last auth key
		if ( ($authkey = $this->parameters_authkey ($key, $matches[3])) === false ) {
			$this->set_error ('\'' . $this->current_key . '\' has invalid auth key');
			return false;
		}

		$hashes = $GLOBALS['___hashunit'][$this->hashtype];
		if ( array_search ($authkey, $hashes[$this->current_type]) === false ) {
			$this->set_error ('\'' . $this->current_key . '\' has invalid variable key' .
								' "' . $this->current_type . '"');
			return false;
		}

		if ( $this->parameters_variable_decode ($value) === false )
			return false;

		return true;
	}
	// }}}

	// {{{ private (boolean) sURI_Parser::parameters_variable_decode (&$value) {
	/**
	 * @access private
	 * @return boolean
	 * @param string
	 */
	private function parameters_variable_decode (&$value) {
		if ( ! is_array ($value) ) {
			if ( function_exists ('get_magic_quotes_gpc') && ini_get ('magic_quotes_gpc') )
				$value = stripslashes ($value);

			while ( preg_match ('/%[0-9a-z]{2}/i', $value) )
				$value = rawurldecode ($value);

			$this->parameters_stripslashes ($value);
			return $this->parameters_variable_confirm ($value);
		} else {
			foreach ( $value as $key => $v ) {
				if ( $this->parameters_variable_decode ($v) === false )
					return false;
				$value[$key] = $v;
			}
		}

		return true;
	}
	// }}}

	// {{{ private (void) sURI_Parser::parameters_stripslashes (&$v)
	/**
	 * remove null byte and directory path
	 *
	 * @access private
	 * @return void
	 * @param string
	 */
	private function parameters_stripslashes (&$v) {
		if ( $this->current_type == 'F' )
			$v = preg_replace ('![\x00]!', '', $v);
		else
			$v = preg_replace ('![\x00]|%(25|5C)?00|\x5c[0]{1,2}|[.]{1,2}/|^/!i', '', $v);
	}
	// }}}

	// {{{ private (boolean) sURI_Parser::parameters_variable_confirm (&$value)
	/**
	 * @access private
	 * @return boolean
	 * @param string
	 */
	private function parameters_variable_confirm (&$value) {
		$keyname = '\'' . $this->current_key . '\'';
		if ( ! $value )
			return true;

		switch ($this->current_type) {
			case 'F' :
				/*
				 * Todo list
				 * Prevent external link code !!!!!!!!!!!!!!!!!
				 */
				if ( $this->allow_url_block && is_array ($this->allow_url_blocks) )
					$this->url_addslashes ($value, $this->allow_url_blocks);
				break;
			case 'S' :
				// change meta characters to htmlentities
				$this->encode_meta_character ($value);
				break;
			case 'a' :
				// check ascii
				if ( preg_match ('/^[^\x01-\x7F]+$/', $value) ) {
					$this->set_error ('Can\'t use out of ascii area value on ' . $keyname);
					return false;
				}

				$this->encode_meta_character ($value);
				break;
			case 'R' :
				// check alphbat || integer
				if ( ! preg_match ('/^[a-z0-9]+$/i', $value) ) {
					$this->set_error ('Can use only alphabat or numeric on ' . $keyname);
					return false;
				}
				break;
			case 'A' :
				// check alphbat
				if ( ! preg_match ('/^[a-z]+$/i', $value) ) {
					$this->set_error ('Can use only Alphabat on ' . $keyname);
					return false;
				}
				break;
			case 'I' :
				// check integer
				if ( ! preg_match ('/^[0-9]+$/', $value) ) {
					$this->set_error ('Can use only numeric on ' . $keyname);
					return false;
				}
				break;
			default :
				$this->set_error ('Unknown error' . $keyname);
				return false;

		}

		return true;
	}
	// }}}

	// {{{ (void) sURI_Parser::encode_meta_character (&$val)
	/**
	 * 메타 문자를 HEX 로 인코딩
	 *
	 * @access public
	 * @return void
	 * @param string
	 */
	public function encode_meta_character (&$val) {
		# pcre 정규식 버그인 듯..
		#$pattern = '/[\x21-\x24\x26-\x2a\x3a-\x3c]|[\2f\x3e\x5b\x5d\x5e]|[\x7b-\x7d]/';
		$pattern = $pattern = '/[!"#$&\'()*\/:;<>\[\]\^{}|]/s';
		$val = preg_replace_callback ($pattern, create_function ('$m', '
										$dec = ord ($m[0]);
										if ( $dec < 100 ) $dec = \'0\' . $dec;
										return \'&#\' . $dec . \';\';'), $val);
	}
	// }}}

	// {{{ (void) sURI_Parser::decode_meta_character (&$val)
	/**
	 * HEX 문자열을 디코딩
	 *
	 * @access public
	 * @return void
	 * @param string
	 */
	public function decode_meta_character (&$val) {
		$pattern = '/&#([0-9]{3});/';
		$val = preg_replace_callback ($pattern, create_function ('$m', 'return chr ($m[1]);'), $val);
	}
	// }}}

	// {{{ (boolean) sURI_Parser::parameters_authkey ($name, $key)
	/**
	 *
	 * @access private
	 * @param string $name => uri parameter name
	 * @param string $key  => uri parameter name auth key
	 */
	private function parameters_authkey ($name, $key) {
		$l = strlen ($key);
		$confirm = 0;

		if ( $l > 3 )
			return false;

		if ( $this->make_name_key ($name) != $key[2] )
			return false;

		if ( ($r = $this->parameters_method ($key, true)) === false )
			return false;

		return $r;
	}
	// }}}

	// {{{ private (string) sURI_Parser::parameters_method ($key, $check = false)
	/**
	 * @access private
	 * @return string
	 * @param string
	 * @param boolean
	 */
	private function parameters_method ($key, $check = false) {
		preg_match_all ('/[PG]/', $key, $matches);
		if ( count ($matches[0]) != 1 )
			return false;

		if ( $check === true ) {
			switch ($matches[0][0]) {
				case 'P' :
					if ( $this->pmethod != 'G' )
						return false;
					break;
				case 'G' :
					if ( $this->pmethod != 'P' )
						return false;
					break;
				default :
					return false;
			}
			$key = $key[0] . $key[1];
		} else
			$this->pmethod = $matches[0][0];

		return preg_replace ('/[PG]/', '', $key);
	}
	// }}}

	// {{{ (void) sURI_Parser::array_is (&$v, $allow_single = false) {
	/**
	 * 주어진 값이 배열인지 여부를 확인하여 배열이 아니면 배열로 변환
	 * @access public
	 * @return void
	 * @param mixed   배열 또는 문자열
	 * @param boolean (optional) true로 설정될 경우 값이 '키=값'으로
	 *                지정되었을 경우 key/value로 배열 생성
	 */
	function array_is (&$v, $allow_single = false) {
		if ( is_array ($v) )
			return;

		if ( ! trim ($v) )
			$v = array ();
		else {
			if ( $allow_single ) {
				$v = array ($v);
			} else {
				if ( preg_match ('/^([^=]+)=(.+)$/', $v, $matches) )
					$v = array ($matches[1] => $matches[2]);
				else
					$v = array ();
			}
		}
	}
	// }}}

    // {{{ (void) sURI_Parser::magic_quotes_gpc_off (void)
	/**
	 * URI 매개변수값의 quote 된 문자를 디코딩
	 *
	 * @access public
	 * @return void
	 */
	public function magic_quotes_gpc_off () {
		if ( ! function_exists (get_magic_quotes_gpc) )
			return;

		if ( ! ini_get ('magic_quotes_gpc') )
			return;

		self::stripslashes_array ($_GET);
		self::stripslashes_array ($_POST);
		self::stripslashes_array ($_COOKIE);
	}
	// }}}

	// {{{ (string) sURI_Parser::bind_hash ($var)
	/**
	 * @access public
	 * @param string $var
	 *
	 *        I  [A-I] int
	 *        A  [J-R] alphabat
	 *        R  [S-a] alphabat & int
	 *        a  [b-j] ascii
	 *        S  [k-s] string
	 *        F  [t-1] full
	 *        N  [2-9] parameter auth_key
	 *        P  .     post
	 *        G  :     get
	 */
	public function bind_hash ($var) {
		global $___hashunit;

		$l = strlen ($var);

		$random = $this->bind_hash_random ($var, true);
		foreach ( $random as $v ) {
			switch ($v) {
				case 'I' :
				case 'A' :
				case 'R' :
				case 'a' :
				case 'S' :
				case 'T' :
				case 'F' :
					$k = array_rand ($___hashunit[$this->hashtype][$v]);
					$r .= $___hashunit[$this->hashtype][$v][$k];
					break;
				default :
					$r .= ($v == 'P') ? 'G' : 'P';
			}
		}

		return $r;
	}
	// }}}

	// {{{ (array|string) sURI_Parser::bind_hash_random ($var, $array_is = false)
	/**
	 * @access public
	 * @return array|string
	 * @param string
	 * @param boolean (optional) 기본값 false
	 */
	public function bind_hash_random ($var, $array_is = false) {
		$arraystr = $this->bind_hash_string_array ($var);
		$key = array_rand ($arraystr, count ($arraystr));

		foreach ( $key as $v ) {
			if ( $array_is )
				$r[] = $var[$v];
			else
				$r .= $var[$v];
		}

		return $r;
	}
	// }}}

	// {{{ (array) sURI_Parser::bind_hash_string_array ($var)
	/**
	 * @access public
	 * @return array
	 * @param string
	 */
	public function bind_hash_string_array ($var) {
		$l = strlen ($var);
		for ( $i=0; $i<$l; $i++ )
			$r[$i] = $var[$i];

		return $r;
	}
	// }}}

    // {{{ private (void) sURI_Parser::magic_quotes_gpc_off (&$v)
	/**
	 * @access private 
	 * @return void
	 * @param array
	 */
	private function stripslashes_array (&$v) {
		if ( ! is_array ($v) )
			return;

		foreach ( $v as $key => $value ) {
			if ( ! is_array ($value) ) {
				$v[$key] = stripslashes ($value);
				// block about null byte attack or path attack
				$v[$key] = preg_replace ('![\x00]|%(25|5C)?00|\x5c[0]{1,2}|[.]{1,2}/|^/!i', '', $v[$key]);
			} else {
				self::stripslashes_array ($value);
				$v[$key] = $value;
			}
		}
	}
	// }}}

    // {{{ (void) sURI_Parser::url_addslashes (&$string, $block = null)
	/**
	 * @access public
	 * @return void
	 * @param string
	 * @param array  (optional) 기본값 null
	 */
	public function url_addslashes (&$string, $block = null) {
		if ( ! is_array ($block) )
			return;

		$blocks = $this->block_devided ($string, $block);
		if ( ! is_array ($blocks) || count ($blocks) < 1 )
			return;

		$string = '';
		foreach ( $blocks as $val )
			$string .= $val;
	}
	// }}}

    // {{{ private (array) sURI_Parser::block_devided ($string, $block = null)
	/**
	 * @access public
	 * @return array
	 * @param string
	 * @param array  (optional) 기본값 null
	 */
	private function block_devided ($string, $block = null) {
		if ( ! is_array ($block) )
			return false;

		while ( ($p = $this->array_pos ($string, $block)) !== false ) {
			if ( $p->pos > 0 ) {
				$r[]= htmlspecialchars (substr ($string, 0, $p->pos - 1));
				$string = substr ($string, $p->pos);
			}

			$end = preg_replace ('/[\[<]/', '\\0/', $p->needle);
			if ( ($pp = strpos ($string, $end)) !== false ) {
				$r[] = substr ($string, 0, $pp + strlen ($end));
				$string = substr ($string, $pp + strlen ($end) + 1);
			} else
				break;
		}

		if ( $string )
			$r[] = htmlspecialchars ($string);

		return $r;
	}
	// }}}

    // {{{ (object) sURI_Parser::array_pos ($haystack, $needle = null)
	/**
	 * @access public
	 * @return stdClass|false
	 *   <pre>
	 *   stdClass Object
	 *   (
	 *       [needle] => adjusted key on needle
	 *       [pos]    => search position
	 *   )
	 *   </pre>
	 *
	 * @param string
	 * @param array  (optional) 기본값 null
	 */
	public function array_pos ($haystack, $needle = null) {
		if ( ! is_array ($needle) )
			return false;

		$p = null;
		$r = (object) array ('needle' => false, 'pos' => false);

		foreach ( $needle as $val )
			$p[] = strpos ($haystack, $val);

		if ( ! is_array ($p) )
			return false;

		$mlen = false;
		$chkkey = 0;

		foreach ( $p as $key => $v ) {
			if ( $v === false )
				continue;

			if ( $mlen !== false ) {
				$mlen = ($mlen > $v) ? $v : $mlen;
				$chkkey = $key;
			} else
				$mlen = $v;
		}

		if ( $mlen === false )
			return false;

		$r->needle = $needle[$chkkey];
		$r->pos = $mlen;

		return $r;
	}
	// }}}
}

?>
