<?php
/**
 * Project: sURI :: Secure URI<br>
 * File:    sURI/Template.php
 *
 * sURI에서 사용하는 HTML Template API
 *
 * @category   HTML
 * @package    sURI
 * @subpackage sURI_template
 * @author     JoungKyun.Kim <http://oops.org>
 * @copyright  (c) 2014 JoungKyun.Kim
 * @license    BSD
 * @version    $Id: Template.php 59 2014-01-25 10:01:51Z oops $
 * @link       http://pear.oops.org/package/sURI
 * @since      File available since Release 0.1
 * @filesource
 */

/**
 * sURI에서 사용하는 HTML Template API
 * @package sURI
 */
Class sURI_Template extends sURI_Dom {
	// {{{ private properties
	/**#@+
	 * @access private
	 */
	private $stdAttr = array (
		'id', 'class', 'title', 'style', 'dir', 'lang',
		'xml:lang', 'accesskey', 'tabindex'
	);
	private $eventAttr = array (
		'onload', 'onunload', 'onchange', 'onsubmit', 'onreset',
		'onselect', 'onblur', 'onfocus', 'onkeydown', 'onkeypress',
		'onkeyup', 'onclick', 'ondbclick', 'onmousedown', 'onmouseover',
		'onmouseout', 'onmouseup'
	);
	private $formAttr = array (
		'accept', 'accept-charset', 'name', 'type'
	);

	private $inputAttr = array (
		'value', 'accept', 'align', 'alt', 'disabled', 'maxlength',
		'readonly', 'size', 'src'
	);

	private $selectAttr = array (
		'disabled', 'multiple', 'size'
	);

	private $textareaAttr = array (
		'readonly', 'disabled', 'wordwrap'
	);

	private $bindv = null;
	private $bindh = null;
	/**#@-*/
	// }}}

	// {{{ public properties
	/**#@+
	 * @access public
	 */
	public $addnewline = false;
	public $cmethod  = 'get';
	public $cmethodv = 'G';

	const INTEGERS  = 0x0000000f;
	const ALPHABATS = 0x000000f0;
	const ASCIIS    = 0x00000f00;
	const STRINGS   = 0x0000f000;
	const FULL      = 0x000f0000;

	const POST      = 0x000f;
	const GET       = 0x00ff;
	/**#@-*/
	// }}}

	// {{{ private (array) sURI::Template::attrset ($v = null)
	/**
	 * @access private
	 * @return array
	 * @param string (optional) 기본값 null
	 */
	private function attrset ($v = null) {
		if ( $v === null )
			return array_merge ($this->stdAttr, $this->eventAttr);

		$name = $v . 'Attr';
		return array_merge ($this->{$name}, $this->stdAttr, $this->eventAttr);
	}
	// }}}

	// {{{ private (string) sURI_Template::action_attr ($type, $attr = null)
	/**
	 * @access private
	 * @return string
	 * @param string
	 * @param array   (optional) 기본값 null
	 */
	private function action_attr ($type, $attr = null) {
		$this->array_is ($attr);
		$filter = $this->attrset ($type);

		foreach ( $attr as $key => $value ) {
			$key = strtolower ($key);
			if ( array_search ($key, $filter) === FALSE ) {
				$this->set_error ('\'' . $key . '\' is invalid attributes of reset');
				return false;
			}
			$buf .= $key . '="' . $value . '" ';
		}

		return $buf;
	}
	// }}}

	// {{{ (boolean) sURI_Template::form ($method, $action = null, $bin = false, $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <form> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string form mehtd 속성을 소문자로 지정
	 * @param string (optional) form action 속성 값. 기본값은 $_SERVER['PHP_SELF']
	 * @param bool   (optional) 폼에서 파일 업로드를 관리할 경우 ture로 설정
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function form ($method, $action = null, $bin = false, $attr = null) {
		$method = strtolower ($method);
		if ( array_search ($method, array ('post', 'get', 'end')) === FALSE ) {
			$this->set_error ('\'' . $method . '\' is invalid value of form method');
			return false;
		}

		if ( $method == 'end' ) {
			echo '</form>';
			$this->newline ();
			$this->cmethod  = 'get';
			$this->cmethodv = 'G';
			return true;
		}

		$action = trim ($action) ? trim ($action) : $_SERVER['PHP_SELF'];

		$this->cmethod  = strtolower ($method);
		$this->cmethodv = ($method == 'get') ? 'G' : 'P';

		$buf = '<form ';
		$buf .= 'method="' . $method . '" ';

		if ( $bin != false )
			$buf .= 'enctype="multipart/form-data" ';

		$attr_buf = $this->action_attr ('form', $attr);
		if ( $attr_buf === false )
			return false;
		$buf .= $attr_buf;

		$buf .= 'action="' . $this->addrenc ($action) . '">';
		$this->output ($buf);

		return true;
	}
	// }}}

	// {{{ (boolean) sUIR_Template::input ($type, $name, $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <input> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string 소문자로 된 input type [file, text, image, password]
	 * @param string input name 속성 값
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function input ($type, $name, $attr = null) {
		$type = strtolower ($type);

		if ( array_search ($type, array ('file', 'text', 'image', 'password')) === FALSE ) {
			$this->set_error ('\'' . $type . '\' is invalid value of input method');
			return false;
		}

		if ( ! $this->name_is ($name) )
			return false;

		if ( ! $this->bindv )
			$this->bind (self::INTEGERS);

		$this->name_enc ($name);

		$buf  = '<input ';
		$buf .= 'type="' . $type . '" ';
		$buf .= 'name="' . $name . '" ';

		$filter = $this->attrset ('input');
		$this->array_is ($attr);
		foreach ($attr as $key => $value) {
			$key = strtolower ($key);
			if ( array_search ($key, $filter) === FALSE ) {
				$this->set_error ('\'' . $key . '\' is invalid attributes of form');
				return false;
			}

			if ( $type != 'image' &&  array_search ($key, array ('src', 'align', 'alt')) !== FALSE ) {
				$this->set_error ('\'' . $key . '\' is only valid attribute on image type');
				return false;
			}

			if ( $type == 'image' && array_search ($key, array ('maxlength', 'readonly')) === FALSE ) {
				$this->set_error ('\'' . $key . '\' is only valid attribute on text or password type');
				return false;
			}

			$value = ($key == 'src' ) ? $this->addrenc ($value) : $value;
			$value = ($key == 'value' ) ? $this->urlenc ($value) : $value;
			$buf .= $key . '="' . $value . '" ';
		}
		$this->tagclose ($buf);
		$this->output ($buf);

		return true;
	}
	// }}}

	// {{{ (boolean) sUIR_Template::file ($name, $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="file"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string input name 속성 값
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function file ($name, $attr = null) {
		if ( ! $this->bindv )
			$this->bind (STRINGS);
		return $this->input ('file', $name, $attr);
	}
	// }}}

	// {{{ (boolean) sUIR_Template::image ($name, $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="file"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string input name 속성 값
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function image ($name, $attr = null) {
		if ( ! $this->bindv )
			$this->bind (STRINGS);
		return $this->input ('image', $name, $attr);
	}
	// }}}

	// {{{ (boolean) sUIR_Template::password ($name, $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="password"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string input name 속성 값
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function password ($name, $attr = null) {
		if ( ! $this->bindv )
			$this->bind (FULL);
		return $this->input ('password', $name, $attr);
	}
	// }}}

	// {{{ (boolean) sUIR_Template::textarea ($name, $value = null, $row = 10, $col = 70, $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <textarea> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string name 속성 값
	 * @param string (optional) textarea 기본 내용
	 * @param integer (optional) rows of textarea
	 * @param integer (optional) columns of textarea
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function textarea ($name, $value = null, $row = 10, $col = 70, $attr = null) {
		if ( ! $this->bindv )
			$this->bind (FULL);

		if ( ! $this->name_is ($name) )
			return false;

		$this->name_enc ($name);

		$buf  = '<textarea ';
		$buf .= 'name="' . $name . '" ';
		$buf .= 'rows="' . $row . '" ';
		$buf .= 'cols="' . $col . '" ';

		$attr_buf = $this->action_attr (textarea, $attr);
		if ( $attr_buf === false )
			return false;
		$buf .= $attr_buf;

		$oldnewline = $this->addnewline;
		$this->addnewline = false;
		$this->tagclose ($buf, false);
		$this->output ($buf);

		if ( $value )
			echo $value;

		$this->addnewline = $oldnewline;
		echo "</textarea>";
		$this->newline();

		return true;
	}
	// }}}

	// {{{ (boolean) sUIR_Template::radio ($name, $value, $msg_prefix = false)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="radio"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string  input name 속성 값<br>
	 *                값이 cbox:로 시작할 경우 checkbox로 생성함
	 * @param array   radio button 값
	 * @param boolean (optional) true로 설정이 되면 value 인자의 첫번째
	 *                배열값을 <input> 태그 앞에 출력
	 */
	function radio ($name, $value, $msg_prefix = false) {
		if ( ! $this->bindv )
			$this->bind (self::INTEGERS);

		$this->array_is ($value, true);
		if ( preg_match ('/^cbox:(.*)/', $name, $matches) ) {
			$name = $matches[1];
			$types = 'checkbox';
		} else {
			$types = 'radio';
		}

		if ( ! $this->name_is ($name) )
			return false;

		$this->name_enc ($name);

		$i = 0;
		foreach ( $value as $v ) {
			if ( is_array ($v) ) {
				$comment = $v[0];
				$val = $v[1] ? $v[1] : $v[0];
				$checked = $v[2] ? 'checked="checked" ' : '';
			} else {
				$val = $v;
				$comment = $v;
				$checked = ($i == 0) ? 'checked="checked" ' : '';
			}

			if ( $msg_prefix )
				echo $comment . ' ';
			$buf = '<input type="' . $types . '" ';
			$buf .= 'name="' . $name . '" ';
			$buf .= 'value="' . $this->urlenc ($val) . '" ';
			$buf .= $checked ? $checked : '';
			$this->tagclose ($buf);

			if ( ! $locate ) {
				$oldnewline = $this->addnewline;
				$this->addnewline = false;
			}
			$this->output ($buf);
			if ( ! $msg_prefix ) {
				echo $comment;
				$this->addnewline = $oldnewline;
				$this->newline ();
			}

			$i++;
		}
		return true;
	}
	// }}}

	// {{{ (boolean) sUIR_Template::select ($name, $value, $attr = null)
	function select ($name, $value, $attr = null) {
		if ( ! $this->bindv )
			$this->bind (self::INTEGERS);

		if ( ! $this->name_is ($name) )
			return false;

		$this->name_enc ($name);

		$buf  = '<select ';
		$buf .= 'name="' . $name . '" ';

		$attr_buf = $this->action_attr ('select', $attr);
		if ( $attr_buf === false )
			return false;
		$buf .= $attr_buf;

		$this->tagclose ($buf, false);
		$this->output ($buf);

		$this->array_is ($value);
		$i = 0;
		$buf = '';
		foreach ( $value as $v ) {
			if ( is_array ($v) ) {
				$comment = $v[0];
				$val = $v[1] ? $v[1] : $v[0];
				$checked = $v[2] ? ' selected="selected"' : '';
			} else {
				$val = $v;
				$comment = $v;
				$checked = ($i == 0) ? ' selected="selected"' : '';
			}

			$buf .= "\t<option value=\"{$val}\"{$checked}>{$comment}</option>\n";

			$i++;
		}
		echo $buf;

		echo "</select>";
		$this->newline();

		return true;
	}
	// }}}

	// {{{ (boolean) sUIR_Template::checkbox ($name, $value, $msg_prefix = false)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="checkbox"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string input value 속성 값
	 * @param array   checkbox value 속성 값
	 * @param boolean (optional) true로 설정이 되면 value 인자의 첫번째
	 *                배열값을 <input> 태그 앞에 출력
	 */
	function checkbox ($name, $value, $msg_prefix = false) {
		return $this->radio ('cbox:' . $name, $value, $msg_prefix);
	}
	// }}}

	// {{{ (boolean) sUIR_Template::button ($value, $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="button"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string input value 속성 값
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function button ($value, $attr = null) {
		if ( ! trim ($value) )
			$value = 'BUTTON';

		$buf = '<input type="button" ';
		$buf .= 'type="' . $value . '" ';

		$attr_buf = $this->action_attr (null, $attr);
		if ( $attr_buf === false )
			return false;
		$buf .= $attr_buf;

		$this->tagclose ($buf);
		$this->output ($buf);

		return true;
	}
	// }}}

	// {{{ (boolean) sUIR_Template::hidden ($name, $value)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="button"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string input name 속성 값
	 * @param string input value 속성 값
	 */
	function hidden ($name, $value) {
		if ( ! $this->bindv )
			$this->bind (STRINGS);

		if ( ! $this->name_is ($name) )
			return false;

		$this->name_enc ($name);

		$buf = '<input type="hidden" ';
		$buf .= 'name="' . $name . '" ';
		$buf .= $value ? 'value="' . $this->urlenc ($value) . '" ' : '';
		$this->tagclose ($buf);
		$this->output ($buf);

		return true;
	}
	// }}}

	// {{{ (boolean) sUIR_Template::reset ($value = '', $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="reset"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string input value 속성 값
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function reset ($value = '', $attr = null) {
		$buf = '<input type="reset" ';
		$buf .= $value ? 'value="' . $value . '" ' : '';

		$attr_buf = $this->action_attr (null, $attr);
		if ( $attr_buf === false )
			return false;
		$buf .= $attr_buf;

		$this->tagclose ($buf);
		$this->output ($buf);

		return true;
	}
	// }}}

	// {{{ (boolean) submit ($name = '', $value = '', $attr = null)
	/**
	 * 동적 매개변수 이름을 이용하여 <input type="submit"> 태그를 생성
	 *
	 * @access public
	 * @return boolean
	 * @param string (optional) input name 속성 값
	 * @param string (optional) input value 속성 값
	 * @param array  (optional) 추가 속성. 기본값 null
	 */
	function submit ($name = '', $value = '', $attr = null) {
		if ( $name )
			$this->name_enc ($name);

		$buf = '<input type="submit" ';
		$buf .= $name ? 'name="' . $name . '" ' : '';
		$buf .= $value ? 'value="' . $value . '" ' : '';

		$attr_buf = $this->action_attr (null, $attr);
		if ( $attr_buf === false )
			return false;
		$buf .= $attr_buf;

		$this->tagclose ($buf);
		$this->output ($buf);

		return true;
	}
	// }}}

	// {{{ (stinrg) sUIR_Template::bind ($v, $type = POST)
	/**
	 * 변수 타입을 바인딩
	 *
	 * @access public
	 * @return string|void
	 * @param int
	 *        self::INTEGER  0x0000000f only [0-9]
	 *        self::ALPHABAT 0x000000f0 only [a-zA-Z]
	 *        self::ASCII    0x00000f00 only ascii area
	 *        self::STRINGS  0x0000f000 0x00 ~ 0xff except meta character and null (parsed null)
	 *        self::FULL     0x000f0000
	 * @param int  POST or GET 상수 값. 기본값 POST 상수 값
	 */
	function bind ($v, $type = POST) {
		if ( $type === GET ) {
			$this->bindget ($v);
			return $v;
		}

		switch ($v) {
			case 0x000fffff :
			case 0x000ffff0 :
			case 0x000fff00 :
			case 0x000ff000 :
			case 0x000f0000 :
				// FULL
				$this->bindv = 'F';
				break;
			case 0x0000ffff :
			case 0x0000fff0 :
			case 0x0000ff00 :
			case 0x0000f000 :
				// STRING
				$this->bindv = 'S';
				break;
			case 0x00000fff :
			case 0x00000ff0 :
			case 0x00000f00 :
				// ASCII
				$this->bindv = 'a';
				break;
			case 0x000000ff :
				// INTEGER & ALPHABAT
				$this->bindv = 'R';
				break;
			case 0x000000f0 :
				// ALPHABAT
				$this->bindv = 'A';
				break;
			default :
				// INTEGER
				$this->bindv = 'I';
		}

		$this->bindh = $this->bind_hash ($this->cmethodv . $this->bindv);
	}
	// }}}

	// {{{ (void) sUIR_Template::unbind (&$key)
	/**
	 * 문자열 키값을 상수값으로 변경
	 *
	 * @access public
	 * @return void
	 * @param string
	 */
	function unbind (&$key) {
		switch ($key) {
			case 'INTEGERS' :
			case 'INTEGER' :
				$key = self::INTEGERS;
				break;
			case 'ALPHABATS' :
			case 'ALPHABAT' :
				$key = self::ALPHABATS;
				break;
			case 'ASCIIS' :
			case 'ASCII' :
				$key = self::ASCIIS;
				break;
			case 'STRINGS' :
			case 'STRING' :
				$key = self::STRINGS;
				break;
			case 'FULL' :
				$key = self::FULL;
				break;
			case 'INTEGER|ALPHABAT' :
			case 'INTEGER|ALPHABATS' :
			case 'INTEGERS|ALPHABAT' :
			case 'INTEGERS|ALPHABATS' :
			case 'ALPHABAT|INTEGER' :
			case 'ALPHABAT|INTEGERS' :
			case 'ALPHABATS|INTEGER' :
			case 'ALPHABATS|INTEGERS' :
				$key = 0x000000ff;
				break;
			case 'NONE' :
				$key = 0x00000000;
				break;
			default :
				$key = self::INTEGERS;
		}
	}
	// }}}

	// {{{ private (void) sUIR_Template::bindget ($v)
	/**
	 * @access private
	 * @return void
	 * @param string
	 */
	private function bindget ($v) {
		if ( ! strlen ($v) )
			return;

		if ( ! preg_match ('/[?]/', $v) )
			return;

		$pos = strpos ($v, '?');
		$_uri = (object) array ();
		$_uri->path = substr ($v, 0, $pos);
		$_uri->query = substr ($v, $pos + 1);

		$this->cmethod  = 'get';
		$this->cmethodv = 'G';

		parse_str ($_uri->query, $uri);
		if ( ! is_array ($uri) )
			return;

		foreach ( $uri as $name => $v ) {
			if ( preg_match ('/^([^:]+):(.+)$/', $name, $matches) ) {
				$key = $matches[1];
				$name = $matches[2];
			} else
				$key = $this->legacy ? 'NONE' : 'INTEGERS';

			$this->unbind ($key);

			if ( $key != 0x00 ) {
				$this->bind ($key);
				$this->name_enc ($name);
			}
			$newuri .= $name . '=' . rawurlencode ($v) . '&';
		}

		$pattern_src = array ('/[&]$/', '/[&]/', '/[&]amp;amp;/');
		$pattern_dst = array ('', '&amp;', '&amp;');
		$newuri = preg_replace ($pattern_src, $pattern_dst, $newuri);

		$v = $_uri->path . '?' . $newuri;
	}
	// }}}

	// {{{ private (boolena) sUIR_Template::name_is ($var)
	/**
	 * @access private
	 * @return boolean
	 * @param string
	 */
	private function name_is ($var) {
		if ( ! trim ($var) ) {
			$this->set_error ('\'name\' attribute is not set');
			return false;
		}

		return true;
	}
	// }}}

	// {{{ (void) sUIR_Template::name_enc (&$name)
	/**
	 * @access public
	 * @return void
	 * @param string
	 */
	function name_enc (&$name) {
		global $___hashunit;

		$prefix = $this->bind_hash_random ($this->cmethodv . $this->bindv);
		$postfix = $this->bindh;

		$name_key = $this->make_name_key ($name);
		$postfix .= $name_key;

		$name_ary = '';
		if ( preg_match ('/(.*)(\[\])$/', $name, $matches) ) {
			$name = $matches[1];
			$name_ary = $matches[2];
		}
		$name = $prefix . '_' . $name . '_' . $postfix . $name_ary;
	}
	// }}}

	// {{{ (string) sUIR_Template::make_name_key ($name)
	/**
	 * @access public
	 * @return string
	 * @param string
	 */
	public function make_name_key ($name) {
		global $___hashunit;

		$key = $this->name_enc_key ();
		$len = count ($name);

		$_key = 0;
		for ( $i=0; $i<$len; $i++ )
			$_key += ord ($name[$i]);

		$key = ($_key + $key) % count ($___hashunit[$this->hashtype]['N']);
		return $___hashunit[$this->hashtype]['N'][$key];
	}
	// }}}

	// {{{ private (int) sUIR_Template::name_enc_key ()
	/**
	 * @access private
	 * @return integer
	 */
	private function name_enc_key () {
		global $___hashunit;
		$_hashes = $___hashunit[$this->hashtype]['N'];

		$key = 0;
		foreach ( $_hashes as $val )
			$key += ord ($val);

		return $key % count ($_hashes);
	}
	// }}}

	// {{{ private (void) sUIR_Template::output ($var)
	/**
	 * @access private
	 * @return void
	 * @param string
	 */
	private function output ($var) {
		#echo preg_replace ('![ ]+(/>)!', '\\1', $var);
		echo $var;
		$this->newline ();
		$this->bindv = null;
		$this->bindh = null;
	}
	// }}}

	// {{{ private (void) sUIR_Template::tagclose (&$var, $selfclose = true)
	/**
	 * @access private
	 * @return void
	 * @param string
	 * @param boolean 
	 */
	private function tagclose (&$var, $selfclose = true) {
		if ( $selfclose === false )
			$var .= '>';
		else
			$var .= $this->xhtmlclose . '>';
	}
	// }}}

	// {{{ private (string) sUIR_Template::urlenc ($var)
	/**
	 * @access private
	 * @return string
	 * @param string
	 */
	private function urlenc ($var) {
		if ( $this->urlencode === false )
			return $var;
		return rawurlencode ($var);
	}
	// }}}

	// {{{ private (string) sURI_Template::addrenc ($var)
	/**
	 * @access private
	 * @return string
	 * @param string
	 */
	private function addrenc ($var) {
		return htmlspecialchars ($var);
	}
	// }}}

	// {{{ private (void) sURI_Template::newline (void)
	/**
	 * @access private
	 * @return string
	 */
	private function newline () {
		if ( $this->addnewline )
			echo "\n";
	}
	// }}}

	// {{{ private (mixed) sURI_Template::self_attribute ($var)
	/**
	 * @access private
	 * @return mixed
	 * @param string 
	 */
	private function self_attribute ($var) {
		$selfs = array ('checked', 'disabled', 'readonly');
		if ( array_search ($var, $selfs) !== FALSE )
			return $var;

		return null;
	}
	// }}}
}
?>
